package cn.com.qmth.stmms.admin.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.admin.utils.HttpUtil;
import cn.com.qmth.stmms.biz.exam.model.DataSync;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.service.DataSyncService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.utils.PictureUrlBuilder;

public class DataSyncThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(DataSyncThread.class);

    private DataSync dataSync;

    private ExamStudentService studentService;

    private ExamService examService;

    private LockService lockService;

    private DataSyncService dataSyncService;

    private Integer pageSize;

    private String baseDir;

    private HttpUtil subjectHttp;

    private HttpUtil studentHttp;

    public static final String PAPER_STRUCT = "paperStructList";

    public static final String SUBJECT_CODE = "subjectCode";

    public static final String SUBJECT_NAME = "subjectName";

    public static final String DATA_LIST = "dataList";

    public static final String START_ID = "startId";

    public static final String NEXT_ID = "nextId";

    public static final String DEFAULT_NULL = "#";

    public static final String SIZE = "size";

    public DataSyncThread(DataSync dataSync, String secretKey, String appId, Integer pageSize, String baseDir,
            LockService lockService, DataSyncService dataSyncService, ExamService examService,
            ExamStudentService studentService) {
        this.dataSync = dataSync;
        this.lockService = lockService;
        this.dataSyncService = dataSyncService;
        this.examService = examService;
        this.studentService = studentService;
        this.pageSize = pageSize;
        this.baseDir = baseDir;
        this.subjectHttp = new HttpUtil(dataSync.getUrl(), secretKey, appId, dataSync.getRootOrgId());
        this.studentHttp = new HttpUtil(dataSync.getUrl(), secretKey, appId, dataSync.getRootOrgId());
    }

    @Override
    public void run() {
        log.info("start data sync for examId=" + dataSync.getExamId());
        try {
            // 获取考试信息
            Exam exam = examService.findById(dataSync.getExamId());
            DataSync sync = dataSyncService.findByExamId(dataSync.getExamId());
            if (sync == null) {
                sync = dataSync;
            }
            JSONObject datas = new JSONObject();
            datas.accumulate("examId", dataSync.getCloudExamId());
            String subjectResult = subjectHttp.httpAction(null, datas.toString());
            JSONObject subjectJson = JSONObject.fromObject(subjectResult);
            JSONArray subjectArray = subjectJson.getJSONArray(PAPER_STRUCT);
            for (int i = 0; i < subjectArray.size(); i++) {
                JSONObject subject = subjectArray.getJSONObject(i);
                String subjectCode = subject.getString(SUBJECT_CODE);
                String subjectName = subject.getString(SUBJECT_NAME);
                Long startId = 0L;
                while (startId != null) {
                    datas.accumulate(SUBJECT_CODE, subjectCode);
                    datas.accumulate(START_ID, startId);
                    datas.accumulate(SIZE, pageSize);
                    String studentResult = studentHttp.httpAction(null, datas.toString());

                    JSONObject studentJson = JSONObject.fromObject(studentResult);

                    Long nextId = studentJson.getLong(NEXT_ID);
                    if (startId == nextId) {
                        startId = null;
                    } else {
                        startId = nextId;
                    }

                    JSONArray studentArray = studentJson.getJSONArray(DATA_LIST);
                    List<ExamStudent> list = new ArrayList<ExamStudent>();
                    for (int j = 0; j < studentArray.size(); j++) {
                        JSONObject student = studentArray.getJSONObject(i);
                        ExamStudent examStudent = new ExamStudent();
                        examStudent.setExamId(dataSync.getExamId());
                        examStudent.setStudentCode(student.getString("studentCode"));
                        examStudent.setName(student.getString("name"));
                        examStudent.setCollege(DEFAULT_NULL);
                        examStudent.setClassName(DEFAULT_NULL);
                        examStudent.setTeacher(DEFAULT_NULL);
                        examStudent.setCampusName(DEFAULT_NULL);
                        examStudent.setSubjectCode(subjectCode);
                        examStudent.setSubjectName(subjectName);
                        examStudent.setExamNumber(student.getString("examNumber"));

                        examStudent.setSchoolId(exam.getSchoolId());
                        examStudent.setAbsent(false);
                        examStudent.setUpload(false);
                        examStudent.setException(false);
                        examStudent.setSliceCount(0);
                        examStudent.setSheetCount(0);
                        examStudent.setObjectiveScore(0d);
                        examStudent.setSubjectiveScore(0d);

                        list.add(examStudent);

                        String answerJson = student.getString("subjectives");
                        File file = new File(baseDir
                                + PictureUrlBuilder.getAnswerJson(exam.getId(), subjectCode, null,
                                        examStudent.getExamNumber()));
                        FileOutputStream fos = new FileOutputStream(file);
                        fos.write(answerJson.getBytes());
                        fos.close();
                    }
                    studentService.batchSave(list);
                    dataSync.setUpdateTime(new Date());
                    dataSync.setNextId(nextId);
                    dataSync.setSubjectCode(subjectCode);
                    dataSyncService.save(dataSync);
                }
            }
            dataSync.setFinished(true);
            dataSyncService.save(dataSync);
        } catch (Exception e) {
            log.error("data sync exception for examId=" + dataSync.getExamId(), e);
        } finally {
            lockService.unlock(LockType.DATA_SYNC, dataSync.getExamId());
            log.info("finish data sync for examId=" + dataSync.getExamId());
        }
    }
}
